from __future__ import print_function
import sys
from os.path import isfile

# Path to the place where the tool is installed
tool_path="/gpfs/projects/bsc32/cs_collaboration/BestIterationCutter/"
# Path to the cutter template
template_path = tool_path + "/templates/cutter_template.xml"

# Read trace name
try:
   trace = sys.argv[1]
except IndexError:
   print("Necessary argument: trace with functions")
   exit(1)
if not isfile(trace):
   print("Argument is not a file")
   exit(1)


# Read trace and load function calls
with open(trace,"r") as f:
   data = []
   for l in f:
      if l.count("60000019"):
         if l.count(":0") == 0:
            dn, process, app, p2, dn2, time, event, value = [int(x) for x in l.split(":")]
            data.append((process, time, value))

# Find number of processes
processes = len(set([x[0] for x in data]))

# Find set of different routines
values = [x[2] for x in data]
value_set = set(values)

# Find frequency of routine call
times_called = []
for v in value_set:
   times_called.append(values.count(v)/processes)

# Find number of time-steps
most_common_freq = None
most_common_freq_times = 0
for v in set(times_called):
   if times_called.count(v) > most_common_freq_times:
      most_common_freq = v
      most_common_freq_times = times_called.count(v)

time_steps = most_common_freq

# Find functions that are called once per time-step
functions_called_once_per_step = []
for v in value_set:
   if values.count(v)/ processes == time_steps:
      functions_called_once_per_step.append(v)


# Find which routine is called in first place
for p,t,v in data:
   if p == 1:
      if v in functions_called_once_per_step:
         first_routine = v
         break
# Find the index of the fastest step
indices = []
for proc in range(1,processes):
   times = []
   for p,t,v in data:
      if p == proc:
         if v == first_routine:
            times.append(t)
   times = [t - min(times) for t in times]
   times = [t/10.**9 for t in times]

   t0 = times[:-1]
   t1 = times[1:]
   step_length = [_x1 - _x0 for _x0,_x1 in zip(t0,t1)]
   step_index = step_length.index(min(step_length))
   indices.append(step_index)

best_step = None
best_step_counter = 0
for step in set(indices):
   if indices.count(step) > best_step_counter:
      best_step = step
      best_step_counter = indices.count(step)


# Find the start and the end of the best step
start = 10**40
end = 0
for proc in range(1,processes):
   counter = 0
   for p,t,v in data:
      if p == proc:
         if v == first_routine:
             counter += 1
             if ( counter == best_step ):
                if t < start:
                   start = t
             if ( counter == best_step + 1):
                if t > end:
                   end = t


if start > 10**39 or end < 1:
   print("Did not find an iteration")
   exit(1)


# Read template
with open(template_path, "r") as template:
   text = template.read()

# Create paramedir cutter file
with open("generated_cutter.xml", "w") as output:
   text =   text.replace("%START%","%i" % (start - 1000) )
   text =   text.replace("%END%","%i" % (end + 1000) )
   output.write(text)
